/* 
*   Function:  build up  internal data structures for netlist 
*
*   Author:    Prof Jason Cong,  Dongmin Xu 
*	       Computer Sci. Dept., UCLA, CA 90095
*
* Copyright (C) 1997 Regents of the University of California
* All rights reserved.
*/ 

/* 
*   allocate memory for all data structure defined in "netlist.h".
*/

#include <malloc.h>
#include "netlist.h"
#include "netlistInt.h"
#include  "extern.h"


Netlist  *getMemNetlist()
{
	Netlist  *p;

	p = (Netlist *) getMemory( sizeof( NETLIST ) );

	p->nc = 0; p->cells = NULL; p->np = 0; p->pads = NULL;
	p->nn = 0; p->nets = NULL;  p->ext = NULL;

	return( p );
}


Netlist_int  *getMemNetlist_int()
{
	Netlist_int  *p;

	p = (Netlist_int *) getMemory( sizeof( NETLIST_INT ) );

	p->np = 0; p->pins = NULL;
	p->ne = 0; p->epins = NULL;
	p->ns = 0; p->shapes = NULL;
	p->nn = 0; p->nodes = NULL;

	return( p );
}


Cell  *getAcell()
{
   Cell   *p;

   if (netlist->cells) {
      if (netlist->nc >= avail_cell_array_size) {
	 avail_cell_array_size += MEM_ARRAY_INC;

         netlist->cells = (Cell *)realloc((Cell *)(netlist->cells),
				avail_cell_array_size * sizeof(CELL) );
      }
   } else {
      avail_cell_array_size = MEM_ARRAY_INC;
      netlist->cells = (Cell *)malloc( avail_cell_array_size * sizeof(CELL) );
   }

   if (netlist->cells == NULL) {
      printf("ERROR: cannot allocate memory in \"getAcell()\"\n"); stop();
   }

   p = &(netlist->cells[netlist->nc]);

   p->name = NULL;  p->index = (netlist->nc)++;  p->type = CELL_TYPE;
   p->x = 0; p->y = 0;  p->ns = 0;   p->shapes = NULL;
   p->np = 0;  p->pins = NULL;  p->nn = 0;  p->nets = NULL;
   p->ext = NULL;

   return p;
}


Cell  *getApad()
{
   Cell   *p;

   if (netlist->pads) {
      if (netlist->np >= avail_pad_array_size) {
	 avail_pad_array_size += MEM_ARRAY_INC;

         netlist->pads = (Cell *)realloc((Cell *)(netlist->pads),
				avail_pad_array_size * sizeof(CELL) );
      }
   } else {
      avail_pad_array_size = MEM_ARRAY_INC;
      netlist->pads = (Cell *)malloc( avail_pad_array_size * sizeof(CELL) );
   }

   if (netlist->pads == NULL) {
      printf("ERROR: cannot allocate memory in \"getApad()\"\n"); stop();
   }

   p = &(netlist->pads[netlist->np]);

   p->name = NULL;  p->index = (netlist->np)++;  p->type = 0;
   p->x = 0; p->y = 0;  p->ns = 0;   p->shapes = NULL;
   p->np = 0;  p->pins = NULL;  p->nn = 0;  p->nets = NULL;
   p->ext = NULL;

   return p;
}


Net  *getAnet()
{
   Net   *p;

   if (netlist->nets) {
      if (netlist->nn >= avail_net_array_size) {
	 avail_net_array_size += MEM_ARRAY_INC;

         netlist->nets = (Net *)realloc((Net *)(netlist->nets),
				avail_net_array_size * sizeof(NET) );
      }
   } else {
      avail_net_array_size = MEM_ARRAY_INC;
      netlist->nets = (Net *)malloc( avail_net_array_size * sizeof(NET) );
   }

   if (netlist->nets == NULL) {
      printf("ERROR: cannot allocate memory in \"getAnet()\"\n"); stop();
   }

   p = &(netlist->nets[netlist->nn]);

   p->name = NULL; p->index = (netlist->nn)++;
   p->np = 0;  p->pins = NULL;  p->ext = NULL;

   return p;
}


Pin  *getApin()
{
   Pin   *p;

   if (netlist_int->pins) {
      if (netlist_int->np >= avail_pin_array_size) {
	 avail_pin_array_size += MEM_ARRAY_INC;

         netlist_int->pins = (Pin *)realloc((Pin *)(netlist_int->pins),
				avail_pin_array_size * sizeof(PIN) );
      }
   } else {
      avail_pin_array_size = MEM_ARRAY_INC;
      netlist_int->pins = (Pin *)malloc( avail_pin_array_size * sizeof(PIN) );
   }

   if (netlist_int->pins == NULL) {
      printf("ERROR: cannot allocate memory in \"getApin()\"\n"); stop();
   }

   p = &(netlist_int->pins[netlist_int->np]);   (netlist_int->np)++;

   p->name = NULL; p->index = 0;  p->x = 0; p->y = 0; p->cell = NULL;
   p->net = NULL; p->ne = 0;  p->epins = NULL;  p->ext = NULL;

   return p;
}


Epin  *getAepin()
{
   Epin   *p;

   if (netlist_int->epins) {
      if (netlist_int->ne >= avail_epin_array_size) {
	 avail_epin_array_size += MEM_ARRAY_INC;

         netlist_int->epins = (Epin *)realloc((Epin *)(netlist_int->epins),
				avail_epin_array_size * sizeof(EPIN) );
      }
   } else {
      avail_epin_array_size = MEM_ARRAY_INC;
      netlist_int->epins = (Epin *)malloc( avail_epin_array_size * sizeof(EPIN) );
   }

   if (netlist_int->epins == NULL) {
      printf("ERROR: cannot allocate memory in \"getAepin()\"\n"); stop();
   }

   p = &(netlist_int->epins[netlist_int->ne]);   (netlist_int->ne)++;

   p->name = NULL; p->index = 0;  p->x = 0; p->y = 0;  p->ext = NULL;

   return p;
}


Shape  *getAshape()
{
   Shape   *p;

   if (netlist_int->shapes) {
      if (netlist_int->ns >= avail_shape_array_size) {
	 avail_shape_array_size += MEM_ARRAY_INC;

         netlist_int->shapes = (Shape *)realloc((Shape *)(netlist_int->shapes),
				avail_shape_array_size * sizeof(SHAPE) );
      }
   } else {
      avail_shape_array_size = MEM_ARRAY_INC;
      netlist_int->shapes = (Shape *)malloc( avail_shape_array_size * sizeof(SHAPE) );
   }

   if (netlist_int->shapes == NULL) {
      printf("ERROR: cannot allocate memory in \"getAshape()\"\n"); stop();
   }

   p = &(netlist_int->shapes[netlist_int->ns]);   (netlist_int->ns)++;
   p->ext = NULL;

   return p;
}


Binary_tree_node  *getAtreeNode()
{
   Binary_tree_node   *p;

   if (netlist_int->nodes) {
      if (netlist_int->nn >= avail_node_array_size) {
	 avail_node_array_size += MEM_ARRAY_INC;

         netlist_int->nodes = (Binary_tree_node *)realloc((Binary_tree_node *)(netlist_int->nodes), avail_node_array_size * sizeof(BINARY_TREE_NODE) );
      }
   } else {
      avail_node_array_size = MEM_ARRAY_INC;
      netlist_int->nodes = (Binary_tree_node *)malloc( avail_node_array_size * sizeof(BINARY_TREE_NODE) );
   }

   if (netlist_int->nodes == NULL) {
      printf("ERROR: cannot allocate memory in \"getAtreeNode()\"\n"); stop();
   }

   p = &(netlist_int->nodes[netlist_int->nn]);   (netlist_int->nn)++;
   p->name = (char *)NULL;  p->index = 0;  p->left = NULL;  p->right = NULL;

   return p;
}


/* getNetPointerArray() get memory for Net pointer array */
Net  **getNetPointerArray(int n)
{
	Net  **p;

	p = (Net **) getMemory( n * sizeof(Net *) );

	return( p );
}


/* getPinPointerArray() get memory for Net pointer array */
Pin  **getPinPointerArray(int n)
{
	Pin  **p;

	p = (Pin **) getMemory( n * sizeof(Pin *) );

	return( p );
}



/*
It gets up a piece of memory according to demand of size. Here
*point is a pointer. remainsize indicates remaining size concerning
with *point. getMemory() returns the available pointer.
*/
char  *getMemory(unsigned size )
{
	char *point;

	point = malloc( size );
	if ( point == NULL )
		stopAt("getMemory","getmem.c");
	return( point );
}


