/* 
*   Function:  build up  internal data structures for netlist 
*
*   Author:    Prof Jason Cong,  Dongmin Xu 
*	       Computer Sci. Dept., UCLA, CA 90095
*
* Copyright (C) 1997 Regents of the University of California
* All rights reserved.
*/ 

/* 
*   Build up  the internal data structures for netlist
*/ 

#include <ctype.h>
#include <string.h>
#include <values.h>
#include <search.h>

#define   USE_IT_ONLY_ONCE
#include "netlist.h"
#include "netlistInt.h"
#include  "extern.h"


read_netlist(char *filename)
{
   FILE   *fp, *initFile();
   char   *st;
   int    endFile;

   fp = initFile(filename, 'r');

   read_netlist_init();			/* initialization */

   endFile = pickupWord(fp,&st);
   while ( endFile ) {
	switch ( st[0] ) {
	case 'h':	/* hard cell */ 
		endFile = readAhardCell(fp, &st);  break;

	case 's':	/* soft cell */ 
		endFile = readAhardCell(fp, &st);  break;

	case 'p': 	/* pad */
		endFile = readApad(fp, &st);  break;
	default:
                stringError(st,"hardcell softcell or pad as keyword"); 
				/* <st> not "cell" or "pad" error */
		break;
	} /* switch */
   } /* while */

   read_netlist_end();			/* close up */

   closeFile(fp);
}


int readAhardCell(FILE *fp, char **st)
{
   Cell  *cell;
   char  *cellName;
   int   cellId, area, endFile=1;
   int   llx, lly, urx, ury;

   cell = getAcell();		/* get a new cell */

   skipNwords(fp, 2);		/* skip ID and "name" */

   pickupWord(fp,st);		/* get cell name */
   cellName = getMemory((unsigned)strlen(*st) + 1);
   strcpy(cellName, *st);   cell->name = cellName;
/*
printf("cell %s: ", cell->name);
*/

   pickupWord(fp,st);		/* get next word */
   readCorners(fp, cell, st);	/* read in a corner */

   endFile = readPins(fp, cell, st);
   					/* read all pins */
   return(endFile);
}


int readPins(FILE *fp, Cell *cell, char **st)
{
   int   endFile;

   if (strNE(*st, "pin"))
      stringError(*st,"pin");	/* <st> not "pin" error */

   while (strEQ(*st,"pin")) { 	/* read pins */
	endFile = readApin(fp, cell, st);
		/* read a pin and return the pin pointer */
   }

   return( endFile );
}


int  readApin(FILE *fp, Cell *cell, char **st)
{
   Pin   *pin;
   char  *pinName;

   pin = getApin();			/* get memory for a pin */

   skipNwords(fp, 1);			/* skip "name" */

   pickupWord(fp, st);			/* get the pinName */
   pinName = getMemory((unsigned)strlen(*st) + 1);
   strcpy(pinName, *st);

   pickupWord(fp,st);			/* get word "signal" */
   if (strNE(*st, "signal"))
      stringError(*st,"signal");	/* <st> not "signal" error */

   readAnet(fp, pin, cell->index, st);
					/* read a net */

   if (!(cell->pins)) {		/* first pin */
      cell->pins = (Pin *)(netlist_int->np);
   }
   pin->index = (cell->np)++;   pin->name = pinName;

   skipNwords(fp, 2);			/* skip "layer 1" */
   pin->x = pickupInt(fp);  pin->y = pickupInt(fp);

/*
printf("pin %s(%d): (x, y) = (%d, %d)\n", pin->name, pin, pin->x, pin->y);
*/

   if (pickupWord(fp, st))			/* get word "pin" */
      return(readEpins(fp, pin, st));
   else
      return 0;
}


int  readAnet(FILE *fp, Pin *pin, int index, char **st)
{
   Net   *net;
   char  *netName;

   pickupWord(fp,st);		/* get signalName */
   netName = getMemory((unsigned)strlen(*st) + 1);
   strcpy(netName, *st);

   if ( !(net=getNetFromHashTable(netName)) ) {		/* not in table */
#if 0
   if ( !(net=getNetFromTreeTable(netName)) ) {		/* not in tree */
#endif
      net = getAnet();			/* get memory for a net */
      net->name = netName;

      putNetInHashTable(net);
#if 0
      putNetInTreeTable(net);
#endif
   } 

   (net->np)++;				/* # of pins in a net */
/*
printf("net=%d, net->np = %d\n", net, net->np);
*/

   pin->net = (Net *)(net->index +1);	/* save index only */
   pin->cell = (Cell *)(index + 1);		/* save index only */
}


/*---------------------------------------------------------------*/
void putNetInHashTable(Net *net)
{
   ENTRY item,  *hsearch();

   item.key = net->name;   item.data = (char *)(net->index);
/*
printf("Put net %s into hash\n", net->name);
*/

   hsearch(item, ENTER);		/* put item into table */
}


Net  *getNetFromHashTable(char *netName)
{
   ENTRY   item, *found_item, *hsearch();
   char    name_to_find[100];

   sprintf(name_to_find, "%s", netName);
   item.key = name_to_find;

   if (found_item = hsearch(item, FIND)) {
/*
printf("--- net %s in hash ALREADY\n", netName);
*/
      return (Net *) (getAnetByIndex((int)(found_item->data)));
   } else {
      return (Net *) NULL;
   }
}


/*---------------------------------------------------------------*/
Net *getNetFromTreeTable(char *name)
{
   Binary_tree_node  *ptr;

   ptr = binary_tree_root;

   while ( ptr != NULL ) {
       switch ( stringcmp(name, ptr->name) ) {

       case 0: 
/*
printf("--- net %s is in binary tree already\n", name);
*/
       		return(getAnetByIndex(ptr->index)); break;

       case 1:  ptr=ptr->right;  break;

       case -1: ptr=ptr->left;   break;
       }
   } /* while */

   return( NULL ); 
}


int putNetInTreeTable(Net *net)
{
   Binary_tree_node  *node,  *pos;

   node = getAtreeNode();		/* get memory for a node */
   node->name = net->name;   node->index = net->index;

/*
printf("Put net %s into binary tree\n", net->name);
*/
   if (binary_tree_root == NULL) {

	binary_tree_root = node;   return 0;	/*sucess*/

   } else if ((pos=getNodeLocation(node->name))!= NULL) {

	switch ( stringcmp(node->name, pos->name) ) {

	case 0: printf("ERROR: net %s is in tree already\n", node->name);
		stop();  break;

	case 1: pos->right = node;  break;

	case -1: 
		pos->left = node;  break;
	}

   } else {
	printf("ERROR: cannot put net %s into binary tree\n", net->name);
	stop();
   }
   return 0;
}


Binary_tree_node *getNodeLocation(char *name)
{
   Binary_tree_node   *ptr,  *ptr_m;

   ptr = binary_tree_root;

   while ( ptr != NULL )  {
	ptr_m = ptr; 

	switch ( stringcmp(name, ptr->name) ) {
	case 0: 
		return( NULL );
	case 1: 
		ptr=ptr->right;  break;
	case -1: 
		ptr=ptr->left;   break;
	}
   }  /* while */

   return( ptr_m );
}


/*---------------------------------------------------------------*/

int  readEpins(FILE *fp, Pin *pin, char **st)
{
   int   endFile=1;

   while (strEQ(*st,"equiv")) { 	/* read pins */
	endFile = readAepin(fp, pin, st);
		/* read a pin and return the pin pointer */
   }

   return( endFile );
}


int  readAepin(FILE *fp, Pin *pin, char **st)
{
   Epin   *epin;
   char  *epinName;
   int   x, y;

   skipNwords(fp, 1);			/* skip "name" */

   pickupWord(fp, st);			/* get the pinName */
   epinName = getMemory((unsigned)strlen(*st) + 1);
   strcpy(epinName, *st);

   epin = getAepin();			/* get memory for a epin */

   if (!(pin->epins)) {			/* first pin */
      pin->epins = (Epin *)(netlist_int->ne);
   }
   epin->index = (pin->ne)++;   epin->name = epinName;

   skipNwords(fp, 2);			/* skip "layer 0" */
   epin->x = pickupInt(fp);  epin->y = pickupInt(fp);

/*
printf("    equiv %s: (x, y) = (%d, %d)\n", epin->name, epin->x, epin->y);
*/

   return( pickupWord(fp, st) ); 	/* get word "pin" */
}


/*---------------------------------------------------------------*/
/* read in shapes */
void readCorners(FILE *fp, Cell *cell, char **st)
{
   while (strEQ(*st, "corners")) {
      readAcorner(fp, cell, st);
   }

   if (strEQ(*st, "class")) {
   	skipNwords(fp, 1); 		/* skip one word */

   	skipNwords(fp, 9); 		/* skip orientation */

        pickupWord(fp, st);			/* get word "pin" */
   }
}


/* read in a corner */
void readAcorner(FILE *fp, Cell *cell, char **st)
{
   Shape  *shape;

   if (strNE(*st, "corners")) 
	stringError(*st,"corners");

   shape = getAshape();		/* get memory for a Shape */

   skipNwords(fp, 1);		/* skip a integer */

   shape->llx = pickupInt(fp);   shape->lly = pickupInt(fp);
   skipNwords(fp, 2); 		/* skip two integers */

   shape->urx = pickupInt(fp);   shape->ury = pickupInt(fp);
   skipNwords(fp, 2); 		/* skip two integers */

   if (!(cell->shapes)) {			/* first pin */
      cell->shapes = (Shape *)(netlist_int->ns);
   }
   (cell->ns)++;
   pickupWord(fp, st);		/* get next key word */

/*
printf("corner: type=%d, llx=%d,lly=%d,urx=%d,ury=%d\n",
	cell->type, shape->llx, shape->lly, shape->urx, shape->ury);
*/
}


/*----------------------------------------------------------*/
int readAsoftCell(FILE *fp, char *st)
{
   printf("st = %s\n", st);
}


int readApad(FILE *fp, char **st)
{
   Cell  *pad;
   char  *padName;
   int   padId, area, endFile=1;
   int   llx, lly, urx, ury;

   pad = getApad();			/* get a new pad */

   skipNwords(fp, 2);			/* skip ID and "name" */

   pickupWord(fp, st);			/* get pad name */
   padName = getMemory((unsigned)strlen(*st) + 1);
   strcpy(padName, *st);   pad->name = padName;
/*
printf("pad %s: ", pad->name);
*/

   pickupWord(fp, st);			/* get next word */
   readCorners(fp, pad, st);		/* read in a corner */

   pad->type = getPadSide(fp, st);	/* get pad side */

   if (strEQ(*st, "sidespace")) {
       skipNwords(fp, 1);   pickupWord(fp, st);
   }
   endFile = readPins(fp, pad, st);
   					/* read all pins */
   return(endFile);
}


/* get pad side */
int getPadSide(FILE *fp, char **st)
{
   char  side;

   if (strNE(*st, "restrict"))
      stringError(*st,"restrict");	/* <st> not "restrict" error */

   skipNwords(fp, 1);			/* skip "side" */

   pickupWord(fp,st);			/* get word for pad side */

   switch (*st[0]) {
   case 'L':	pickupWord(fp, st);  return LEFT_PAD;    break;
   case 'R':	pickupWord(fp, st);  return RIGHT_PAD;   break;
   case 'T':	pickupWord(fp, st);  return TOP_PAD;     break;
   case 'B':	pickupWord(fp, st);  return BOTTOM_PAD;  break;

   default:
	printf("ERROR: wrong pad side %c\n", *st[0]);  stop();  break;
   }
}


/*---------------------------------------------------------------*/
/* setup correct pointers for arrays */
setup_arrays()
{
   setup_cell_arrays();

   setup_pad_arrays();
}


/* setup arrays for all cells */
setup_cell_arrays()
{
   Cell  *cell;
   int   i;

   if (netlist->cells) {
      for (i=0;  i< (netlist->nc); i++) {

	  cell = &(netlist->cells[i]);

          cell->shapes = &(netlist_int->shapes[(int)(cell->shapes) - 1]);

	  cell->pins = &(netlist_int->pins[(int)(cell->pins) - 1]);

	  setup_epins_arrays(cell);
      }
   }
}


/* setup arrays for all pads */
setup_pad_arrays()
{
   Cell  *pad;
   int   i;

   if (netlist->pads) {
      for (i=0;  i< (netlist->np); i++) {

	  pad = &(netlist->pads[i]);

          pad->shapes = &(netlist_int->shapes[(int)(pad->shapes) - 1]);

	  pad->pins = &(netlist_int->pins[(int)(pad->pins) - 1]);

	  setup_epins_arrays(pad);
      }
   }
}


setup_epins_arrays(Cell *cell)
{
   Pin   *pin;
   int   i;

   if (cell->pins) {
      for (i=0;  i< (cell->np); i++) {
          pin = &(cell->pins[i]);

          pin->epins = &(netlist_int->epins[(int)(pin->epins) - 1]);

	  if (isAcell(cell))
             pin->cell = &(netlist->cells[(int)(pin->cell) - 1]);
	  else		/* pad */
             pin->cell = &(netlist->pads[(int)(pin->cell) - 1]);

          pin->net = &(netlist->nets[(int)(pin->net) - 1]);
      }
   }
}



/*---------------------------------------------------------------*/
/* initialization */
read_netlist_init()
{
   netlist = getMemNetlist();		/* get memory for netlist */
   
   netlist_int = getMemNetlist_int();	/* get memory for internal use */

   (void) hcreate(MAX_HASH_SIZE);	/* create hash table */
}


/*---------------------------------------------------------------*/
/* close up */
read_netlist_end()
{
   hdestroy();			/* destory hash table */

   setup_arrays();		/* setup correct pointers for arrays */

   setupNetPins();		/* setup pin array for every nets */

   setupCellNets();		/* setup net array for every cells */
}



/* setup net array for every cells */
setupCellNets()
{
   int  i;

   if (netlist->cells) {
      for (i=0;  i< (netlist->nc); i++) {
          setupAcellNets( &(netlist->cells[i]) );
      }
   }
}


setupAcellNets(Cell *cell)
{
   int   i;
   Pin   *pin;

   if (cell) {
      cell->nets = getNetPointerArray(cell->np);   cell->nn = 0;  

      if (cell->pins) {
         for (i=0;  i< (cell->np); i++) {
             pin = &(cell->pins[i]);

	     cell->nets[(cell->nn)++] = pin->net;
         }
      }
   }
}


/* setup pin array for every nets */
setupNetPins()
{
   int  i;

   if (netlist_int->pins) {
      for (i=0;  i< (netlist_int->np); i++) {

          setupAnetPins( &(netlist_int->pins[i]) );
      }
   }
}


setupAnetPins(Pin *pin)
{
   Net  *net;

   net = pin->net;
   if (net != NULL) {
      if ( !(net->pins)) {		/* empty */
	 net->pins = getPinPointerArray(net->np);   net->np = 0;
      }
      net->pins[(net->np)++] = pin;

   } else {
      printf("ERROR: not net for pin %s\n", pin->name);  stop();
   }
}

