#include "netlist.h"


/* traverse netlist */
printNetlist()
{
   printCells();			/* print all cells */

   printPads();				/* print all pads */

   printNets();				/* print all nets */
}



/************************************************************/
/* tranverse all nets */
printNets()
{
   int  i;

   printf("\n--------------- All nets ------------------\n");
   if (netlist->nets) {
      for (i=0;  i< (netlist->nn); i++) {
          printAnet( &(netlist->nets[i]) );
      }
   }
}


printAnet(Net *net)
{
   if (net) {
      printf("net %s: index=%d, np=%d\n",
		net->name, net->index, net->np);
	 
      printAnetPins(net);	/* print all pins of the net */
   }
}


/* print all pins of the net */
printAnetPins(Net *net)
{
   int  i;

   if (net->pins) {
      for (i=0;  i< (net->np); i++) {
          printApin(net->pins[i]);
      }
   }
}


/************************************************************/
/* tranverse all pads */
printPads()
{
   int  i;

   printf("\n--------------- All pads ------------------\n");
   if (netlist->pads) {
      for (i=0;  i< (netlist->np); i++) {
          printApad( &(netlist->pads[i]) );
      }
   }
}


printApad(Cell *pad)
{
   if (pad) {
      printf("pad %s: index=%d, type=%d\n", pad->name, pad->index, pad->type);
	 
      printShapes(pad);			/* print cell shapes */

      printPins(pad);			/* print cell pins */
   }
}



/************************************************************/
/* tranverse all cells */
printCells()
{
   int  i;

   printf("\n--------------- All cells ------------------\n");
   if (netlist->cells) {
      for (i=0;  i< (netlist->nc); i++) {
          printAcell( &(netlist->cells[i]) );
      }
   }
}


printAcell(Cell *cell)
{
   if (cell) {
      printf("cell %s: index=%d, type=%d, ns=%d, np=%d, nn=%d\n",
	cell->name, cell->index, cell->type, cell->ns, cell->np, cell->nn);
	 
      printShapes(cell);		/* print cell shapes */

      printPins(cell);			/* print cell pins */

#if 1
      printNetsBrief( cell->nets, cell->nn);	/* print cell pins */
#endif
   }
}


/* print cell pins */
printNetsBrief(Net **nets, int n)
{
   int  i;
   Net  *net;

   printf("\n");
   for (i=0; i<n; i++) {
      net = nets[i];

      printf("   net %s: index=%d\n", net->name, net->index);
   }
}


/* print cell pins */
printPins(Cell *cell)
{
   int  i;

   if (cell->pins) {
      for (i=0;  i< (cell->np); i++) {
          printApin( &(cell->pins[i]) );
      }
   }
}


printApin(Pin *pin)
{
   if (isAcell(pin->cell)) 
      printf("   pin %s: (x, y) = (%d, %d), cell %s, net %s\n",
	  pin->name, pin->x, pin->y, pin->cell->name, pin->net->name);
   else
      printf("   pin %s: (x, y) = (%d, %d), pad %s, net %s\n",
	  pin->name, pin->x, pin->y, pin->cell->name, pin->net->name);

   printEpins(pin);			/* print equivalent pins */
}


/* print equivalent pins */
printEpins(Pin *pin)
{
   int  i;

   if (pin->epins) {
      for (i=0;  i< (pin->ne); i++) {
          printAepin( &(pin->epins[i]) );
      }
   }
}

printAepin(Epin *epin)
{
   printf("      epin %s: (x, y) = (%d, %d)\n", epin->name, epin->x, epin->y);
}


/* print cell shapes */
printShapes(Cell *cell)
{
   int  i;

   if (cell->shapes) {
      for (i=0;  i< (cell->ns); i++) {
          printAshape( &(cell->shapes[i]) );
      }
   }
}


printAshape(Shape *shape)
{
    printf("   shape: (llx=%d, lly=%d), (urx=%d, ury=%d)\n",
		shape->llx, shape->lly, shape->urx, shape->ury);
}

