/* 
*   Function:  build up  internal data structures for netlist 
*
*   Author:    Prof. Jason Cong
*	       Computer Sci. Dept., UCLA, CA 90095
*
* Copyright (C) 1997 Regents of the University of California
* All rights reserved.
*/ 

#include <stdio.h>

#include "const.h"
#include "macro.h"

typedef struct netlist NETLIST, Netlist;
typedef struct cell CELL, Cell;
typedef struct pin  PIN,  Pin;
typedef struct epin EPIN,  Epin;
typedef struct net  NET,  Net;
typedef struct shape  SHAPE,  Shape;


struct netlist
{
   int   nc;		/* # of cells in the circuit */
   Cell  *cells;	/* cell array */

   int   np;		/* # of pads in the circuit */
   Cell  *pads;		/* pad array */

   int   nn;		/* # of nets in the circuit */
   Net   *nets;		/* net array */

   char  *ext;		/* for extension */
};


struct cell
{
   char  *name;		/* name of cell */
   int   index;		/* index of this cell */

   int   type;		/* with value of CELL_TYPE / LEFT_PAD /
				RIGHT_PAD / TOP_PAD / BOTTOM_PAD */

   int   x, y;		/* center coordinate of this cell */
   int   orientation;	/* orientation of this cell */

   int   ns;		/* # of different shapes */
   Shape *shapes;	/* shape array */

   int   np;		/* the # of pins of this cell */
   Pin   *pins;		/* pin array */

   int   nn;		/* the # of nets connected to this cell */
   Net   **nets;		/* net pointer array */

   char  *ext;		/* for extension */
};


struct shape
{
   int   llx,  lly;	/* lower left corner */
   int   urx,  ury;	/* upper right corner */

   char  *ext;		/* for extension */
};


struct pin
{
   char  *name;		/* pin name */
   int   index;		/* index of this pin */
   int   x, y;		/* position of this pin in the cell */

   Cell  *cell;		/* point to the cell of this pin */
   Net   *net;		/* point to the net the pin belongs to */

   int   ne;		/* # of equivalent pins of this pin */
   Epin  *epins;	/* equivalent pin array */

   char  *ext;		/* for extension */
};


struct epin		/* define equivalent pin */
{
   char  *name;		/* the name of this equivalent pin */
   int   index;		/* index of this epin */
   int   x, y;		/*position of this equivalent pin in the cell*/

   char  *ext;		/* for extension */
};


struct net
{
   char *name;		/* net name */
   int   index;		/* index of this net */

   int  np;		/* # of pins in this net */
   Pin  **pins; 	/* pin pointer array */

   char  *ext;		/* for extension */
};


#ifndef  USE_IT_ONLY_ONCE

extern  Netlist   *netlist;		/* hold the netlist */

#else

Netlist   *netlist=NULL;		/* hold the netlist */

#endif

